@@ -1,10 +1,11 @@
-;;; google-translate-core-ui.el --- google translate core UI
+;;; google-translate-core-ui.el --- The google translate core UI
 
 ;; Copyright (C) 2012 Oleksandr Manzyuk <manzyuk@gmail.com>
 
 ;; Author: Oleksandr Manzyuk <manzyuk@gmail.com>
 ;; Maintainer: Andrey Tykhonov <atykhonov@gmail.com>
 ;; URL: https://github.com/atykhonov/google-translate
+;; Package-Requires: ((emacs "24.3") (popup "0.5.8"))
 ;; Keywords: convenience
 
 ;; Contributors:
@@ -166,10 +167,12 @@
 ;;; Code:
 ;;
 
-(eval-when-compile (require 'cl))
+(eval-when-compile (require 'cl-lib))
 (require 'google-translate-core)
 (require 'ido)
-
+(require 'popup)
+(require 'color)
+(require 'facemenu)
 
 (defvar google-translate-supported-languages-alist
   '(("Afrikaans"           . "af")
@@ -285,7 +288,7 @@ query parameter in HTTP requests.")
 (defvar google-translate-translation-listening-debug nil
   "For debug translation listening purposes.")
 
-(defstruct gtos
+(cl-defstruct gtos
   "google translate output structure contains miscellaneous
   information which intended to be outputed to the buffer, echo
   area or popup tooltip."
@@ -298,8 +301,7 @@ query parameter in HTTP requests.")
   :group 'processes)
 
 (defcustom google-translate-enable-ido-completion nil
-  "If non-NIL, use `ido-completing-read' rather than
-  `completing-read' for reading input."
+  "If non-NIL, use `ido-completing-read' rather than `completing-read' for reading input."
   :group 'google-translate-core-ui
   :type  '(choice (const :tag "No"  nil)
                   (other :tag "Yes" t)))
@@ -312,11 +314,12 @@ query parameter in HTTP requests.")
 
 (defcustom google-translate-listen-program
   (executable-find "mplayer")
-  "The program to use to listen translations. By default the
-program looks for `mplayer' in the PATH, if `mplayer' is found
-then listening function will be available and you'll see `Listen'
-button in the buffer with the translation. You can use any other
-suitable program."
+  "The program to use to listen translations.
+
+By default the program looks for `mplayer' in the PATH, if
+`mplayer' is found then listening function will be available and
+you'll see `Listen' button in the buffer with the translation.
+You can use any other suitable program."
   :group 'google-translate-core-ui
   :type '(string))
 
@@ -327,12 +330,11 @@ suitable program."
 - If it is `nil', output to temporary pop up buffer (default).
 - `echo-area': output to the Echo Area.
 - `popup': output to the popup tooltip using `popup' package.
-- `kill-ring': the output will be added in kill-ring.
+- `kill-ring': the output will be added in `kill-ring'.
 - `current-buffer': the output will be inserted to current buffer.
 - `help': output to help buffer.
 - `paragraph-overlay': output in current buffer overlay paragraph by paragraph.
-- `paragraph-insert': output will be inserted in current buffer paragraph by paragraph.
-"
+- `paragraph-insert': output will be inserted in buffer paragraph by paragraph."
   :group 'google-translate-core-ui
   :type '(repeat (choice (const :tag "temporary popup buffer" nil)
                          (const :tag "Echo Area" echo-area)
@@ -345,10 +347,14 @@ suitable program."
 
 (defcustom google-translate-pop-up-buffer-set-focus
   nil
-  "Determines whether window (buffer) with translation gets focus
-when it pop ups. If `nil', it doesn't get focus and focus remains
-in the same window as was before translation. If `t',
-window (buffer with translation) gets focus.")
+  "Determines whether result window (buffer) gets focus when it pop ups.
+
+If nil, it doesn't get focus and focus remains in the same
+window as was before translation. If t, window (buffer with
+translation) gets focus.")
+
+(defcustom google-translate-display-translation-phonetic t
+  "Determines whether display phonetic transcription of the translating text.")
 
 (defcustom google-translate-listen-button-label
   "[Listen]"
@@ -356,6 +362,21 @@ window (buffer with translation) gets focus.")
   :group 'google-translate-core-ui
   :type 'string)
 
+(defcustom google-translate-translation-to-kill-ring nil
+  "Add translation to kill-ring after translate commands if it's `t'."
+  :type 'boolean
+  :safe #'booleanp
+  :group 'google-translate-core-ui)
+
+(define-derived-mode google-translate-mode
+  help-mode "Google Translate"
+  "Major mode for google translate output.")
+
+(defvar google-translate-mode-mode-map
+  (let ((map (make-sparse-keymap)))
+    map)
+  "Keymap for `google-translate-mode'.")
+
 (defface google-translate-text-face
   '((t (:inherit default)))
   "Face used to display the original text."
@@ -387,11 +408,13 @@ window (buffer with translation) gets focus.")
   :group 'google-translate-core-ui)
 
 (defvar google-translate-input-method-auto-toggling nil
-  "When `t' the current source language is compared with the
-values from `google-translate-preferable-input-methods-alist' and
-enables appropriate input method for the minibuffer. So this
-feature may allow to avoid switching between input methods while
-translating using different languages.")
+  "When t, the current source language is compared.
+
+Compared with the values from
+`google-translate-preferable-input-methods-alist' and enables
+appropriate input method for the minibuffer. So this feature may
+allow to avoid switching between input methods while translating
+using different languages.")
 
 (defvar google-translate-preferable-input-methods-alist
   '((nil . nil))
@@ -414,6 +437,10 @@ In this way, `ukrainian-programmer-dvorak' will be auto enabled
 for the minibuffer when Russian or Ukrainian (as source language)
 is active.")
 
+
+(defvar google-translate-result-translation nil
+  "The result translation of `google-translate-translate'.")
+
 (defun google-translate-supported-languages ()
   "Return a list of names of languages supported by Google Translate."
   (mapcar #'car google-translate-supported-languages-alist))
@@ -425,15 +452,14 @@ is active.")
     (cdr (assoc language google-translate-supported-languages-alist))))
 
 (defun google-translate-language-display-name (abbreviation)
-  "Return a name suitable for use in prompts of the language whose
-abbreviation is ABBREVIATION."
+  "Return a name suitable for use in prompts of the language whose abbreviation is ABBREVIATION."
   (if (string-equal abbreviation "auto")
       "unspecified language"
     (car (rassoc abbreviation google-translate-supported-languages-alist))))
 
 (defun google-translate-paragraph (text face &optional output-format)
-  "Return TEXT as a filled paragraph into the current buffer and
-apply FACE to it. Optionally use OUTPUT-FORMAT."
+  "Return TEXT as a filled paragraph into the current buffer.
+And apply FACE to it. Optionally use OUTPUT-FORMAT."
   (let ((beg (point))
         (output-format
          (if output-format output-format "\n%s\n")))
@@ -524,25 +550,25 @@ source and target languages."
                                                                     format2)
   "Return detailed translation."
   (with-temp-buffer
-    (loop for item across detailed-translation do
-          (let ((index 0)
-                (label (aref item 0)))
-            (unless (string-equal label "")
-              (put-text-property 0 (length label)
-                                 'font-lock-face
-                                 'google-translate-translation-face
-                                 label)
-              (insert (format format1 label))
-              (loop for translation across (aref item 2) do
-                    (let ((content
-                           (format "%s (%s)"
-                                   (aref translation 0)
-                                   (mapconcat 'identity
-                                              (aref translation 1)
-                                              ", "))))
-                      (insert (format format2
-                                      (incf index)
-                                      content)))))))
+    (cl-loop for item across detailed-translation do
+             (let ((index 0)
+                   (label (aref item 0)))
+               (unless (string-equal label "")
+                 (put-text-property 0 (length label)
+                                    'font-lock-face
+                                    'google-translate-translation-face
+                                    label)
+                 (insert (format format1 label))
+                 (cl-loop for translation across (aref item 2) do
+                          (let ((content
+                                 (format "%s (%s)"
+                                         (aref translation 0)
+                                         (mapconcat 'identity
+                                                    (aref translation 1)
+                                                    ", "))))
+                            (insert (format format2
+                                            (cl-incf index)
+                                            content)))))))
     (buffer-substring (point-min) (point-max))))
 
 (defun google-translate--detailed-definition (detailed-definition definition
@@ -552,27 +578,27 @@ source and target languages."
   (with-temp-buffer
     (let ((section "DEFINITION"))
       (put-text-property 0 (length section)
-			 'font-lock-face
-			 'google-translate-translation-face
-			 section)
+                         'font-lock-face
+                         'google-translate-translation-face
+                         section)
       (insert (format "\n%s\n" section)))
-    (loop for item across detailed-definition do
-          (let ((index 0)
-                (label (aref item 0)))
-            (unless (string-equal label "")
-              (put-text-property 0 (length label)
-                                 'font-lock-face
-                                 'google-translate-translation-face
-                                 label)
-              (insert (format format1 label))
-              (loop for definition across (aref item 1) do
-                    (insert (format format2
-                                    (incf index)
-                                    (if (> (length definition) 2)
-                                        (format "%s\n    \"%s\""
-                                                (aref definition 0)
-                                                (aref definition 2))
-                                      (format "%s" (aref definition 0)))))))))
+    (cl-loop for item across detailed-definition do
+             (let ((index 0)
+                   (label (aref item 0)))
+               (unless (string-equal label "")
+                 (put-text-property 0 (length label)
+                                    'font-lock-face
+                                    'google-translate-translation-face
+                                    label)
+                 (insert (format format1 label))
+                 (cl-loop for definition across (aref item 1) do
+                          (insert (format format2
+                                          (cl-incf index)
+                                          (if (> (length definition) 2)
+                                              (format "%s\n    \"%s\""
+                                                      (aref definition 0)
+                                                      (aref definition 2))
+                                            (format "%s" (aref definition 0)))))))))
     (buffer-substring (point-min) (point-max))))
 
 (defun google-translate--suggestion (gtos)
@@ -654,7 +680,9 @@ About the OUTPUT-DESTINATION, check out option
 
 To deal with multi-line regions, sequences of white space
 are replaced with a single space. If the region contains not text, a
-message is printed."
+message is printed.
+
+At last will save result translation to `google-translate-result-translation'."
   (let* ((json (google-translate-request source-language
                                          target-language
                                          text)))
@@ -672,7 +700,8 @@ message is printed."
                :text text
                :text-phonetic (google-translate-json-text-phonetic json)
                :translation (google-translate-json-translation json)
-               :translation-phonetic (google-translate-json-translation-phonetic json)
+               :translation-phonetic (if google-translate-display-translation-phonetic
+                                         (google-translate-json-translation-phonetic json) "")
                :detailed-translation detailed-translation
                :detailed-definition detailed-definition
                :suggestion (when (null detailed-translation)
@@ -702,7 +731,10 @@ message is printed."
          ((equal output-destination 'paragraph-overlay)
           (google-translate-paragraph-overlay-output-translation gtos))
          ((equal output-destination 'paragraph-insert)
-          (google-translate-paragraph-insert-output-translation gtos)))))))
+          (google-translate-paragraph-insert-output-translation gtos)))
+        (setq google-translate-result-translation (gtos-translation gtos))
+        (when google-translate-translation-to-kill-ring
+          (kill-new google-translate-result-translation))))))
 
 (defun google-translate-popup-output-translation (gtos)
   "Output translation to the popup tooltip using `popup'
@@ -769,7 +801,9 @@ http://www.gnu.org/software/emacs/manual/html_node/elisp/The-Echo-Area.html)"
   "Output translation below the paragraph with overlay."
   (let ((start (save-excursion (start-of-paragraph-text) (point)))
         (end (save-excursion (end-of-paragraph-text) (point)))
-        (below-paragraph (save-excursion (end-of-paragraph-text) (next-line) (point)))
+        (below-paragraph (if (eq major-mode 'org-mode)
+                             (save-excursion (previous-line) (point))
+                           (save-excursion (end-of-paragraph-text) (forward-line) (point))))
         (translation (gtos-translation gtos)))
     (with-silent-modifications
       (put-text-property
@@ -782,7 +816,7 @@ http://www.gnu.org/software/emacs/manual/html_node/elisp/The-Echo-Area.html)"
   "Insert translation below the paragraph with overlay."
   (let ((start (save-excursion (start-of-paragraph-text) (point)))
         (end (save-excursion (end-of-paragraph-text) (point)))
-        (below-paragraph (save-excursion (end-of-paragraph-text) (next-line) (point)))
+        (below-paragraph (save-excursion (end-of-paragraph-text) (forward-line) (point)))
         (translation (gtos-translation gtos)))
     (goto-char below-paragraph)
     (insert (concat "\n" translation "\n"))))
@@ -834,7 +868,8 @@ http://www.gnu.org/software/emacs/manual/html_node/elisp/The-Echo-Area.html)"
          (google-translate--detailed-definition
           detailed-definition translation
           "\n%s\n" "%2d. %s\n")
-       ""))))
+       ""))
+    (google-translate-mode)))
 
 (defun google-translate-read-source-language (&optional prompt)
   "Read a source language, with completion, and return its abbreviation.
